@@ -1,13 +1,34 @@
 #ifndef _CONFIG_H__
 #define _CONFIG_H__ 1
 
-typedef struct _Config Config;
+#include <Evas.h>
 
+typedef struct _Config Config;
+typedef struct _Config_Color Config_Color;
+typedef struct _Config_Keys Config_Keys;
+
+struct _Config_Keys
+{
+   const char *keyname;
+   Eina_Bool ctrl;
+   Eina_Bool alt;
+   Eina_Bool shift;
+   Eina_Bool win;
+   Eina_Bool meta;
+   Eina_Bool hyper;
+   const char *cb;
+};
 /* TODO: separate config per terminal (tab, window) and global. */
 
+struct _Config_Color
+{
+   unsigned char r, g, b, a;
+};
+
 struct _Config
 {
-   int version;
+   int               version;
+   int               scrollback;
    struct {
       const char    *name;
       const char    *orig_name; /* not in EET */
@@ -27,20 +48,39 @@ struct _Config
    } helper;
    const char       *theme;
    const char       *background;
-   const char       *wordsep;
-   int               scrollback;
+   double            tab_zoom;
    int               vidmod;
+   int               opacity;
+   int               cg_width;
+   int               cg_height;
    Eina_Bool         jump_on_keypress;
    Eina_Bool         jump_on_change;
    Eina_Bool         flicker_on_key;
    Eina_Bool         disable_cursor_blink;
    Eina_Bool         disable_visual_bell;
+   Eina_Bool         bell_rings;
+   Eina_Bool         active_links;
    Eina_Bool         translucent;
    Eina_Bool         mute;
+   Eina_Bool         visualize;
    Eina_Bool         urg_bell;
    Eina_Bool         multi_instance;
-   
+   Eina_Bool         xterm_256color;
+   Eina_Bool         erase_is_del;
+   Eina_Bool         custom_geometry;
+   Eina_Bool         drag_links;
+   Eina_Bool         login_shell;
+   Eina_Bool         mouse_over_focus;
+   Eina_Bool         colors_use;
+   Eina_Bool         gravatar;
+   Eina_Bool         notabs;
+   Eina_Bool         mv_always_show;
+   Eina_Bool         ty_escapes;
+   Config_Color      colors[(4 * 12)];
+   Eina_List        *keys;
+
    Eina_Bool         temporary; /* not in EET */
+   Eina_Bool         font_set; /* not in EET */
    const char       *config_key; /* not in EET, the key that config was loaded */
 };
 
@@ -51,6 +91,7 @@ void config_save(Config *config, const char *key);
 Config *config_load(const char *key);
 Config *config_fork(Config *config);
 void config_del(Config *config);
+void config_default_font_set(Config *config, Evas *evas);
 
 const char *config_theme_path_get(const Config *config);
 const char *config_theme_path_default_get(const Config *config);
