@@ -3,12 +3,14 @@
 #include <Elementary.h>
 #include "options.h"
 #include "options_font.h"
-#include "options_helpers.h"
-#include "options_behavior.h"
-#include "options_video.h"
 #include "options_theme.h"
 #include "options_wallpaper.h"
 #include "options_colors.h"
+#include "options_video.h"
+#include "options_behavior.h"
+#include "options_keys.h"
+#include "options_helpers.h"
+#include "options_elm.h"
 #include "config.h"
 #include "termio.h"
 
@@ -19,77 +21,46 @@ static Eina_Bool op_out = EINA_FALSE;
 static Ecore_Timer *op_del_timer = NULL;
 static Evas_Object *saved_win = NULL;
 static Evas_Object *saved_bg = NULL;
-static int mode = -1;
+static void (*op_donecb) (void *data) = NULL;
+static void *op_donedata = NULL;
+
+static enum option_mode {
+     OPTION_NONE = 0,
+     OPTION_FONT,
+     OPTION_THEME,
+     OPTION_WALLPAPER,
+     OPTION_COLORS,
+     OPTION_VIDEO,
+     OPTION_BEHAVIOR,
+     OPTION_KEYS,
+     OPTION_HELPERS,
+     OPTION_ELM
+} _mode = 0;
 
 static void
-_cb_op_font(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event __UNUSED__)
+_cb_op(void *data, Evas_Object *obj EINA_UNUSED, void *event EINA_UNUSED)
 {
-   if (mode == 1) return;
-   mode = 1;
+   enum option_mode mode = (intptr_t) data;
+
+   if (_mode == mode) return;
+   _mode = mode;
    edje_object_signal_emit(saved_bg, "optdetails,hide", "terminology");
 }
 
 static void
-_cb_op_theme(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event __UNUSED__)
-{
-   if (mode == 2) return;
-   mode = 2;
-   edje_object_signal_emit(saved_bg, "optdetails,hide", "terminology");
-}
-
-static void
-_cb_op_wallpaper(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event __UNUSED__)
-{
-   if (mode == 3) return;
-   mode = 3;
-   edje_object_signal_emit(saved_bg, "optdetails,hide", "terminology");
-}
-
-static void
-_cb_op_colors(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event __UNUSED__)
-{
-   if (mode == 4) return;
-   mode = 4;
-   edje_object_signal_emit(saved_bg, "optdetails,hide", "terminology");
-}
-
-static void
-_cb_op_video(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event __UNUSED__)
-{
-   if (mode == 5) return;
-   mode = 5;
-   edje_object_signal_emit(saved_bg, "optdetails,hide", "terminology");
-}
-
-static void
-_cb_op_behavior(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event __UNUSED__)
-{
-   if (mode == 6) return;
-   mode = 6;
-   edje_object_signal_emit(saved_bg, "optdetails,hide", "terminology");
-}
-
-static void
-_cb_op_helpers(void *data __UNUSED__, Evas_Object *obj __UNUSED__, void *event __UNUSED__)
-{
-   if (mode == 7) return;
-   mode = 7;
-   edje_object_signal_emit(saved_bg, "optdetails,hide", "terminology");
-}
-
-static void
-_cb_op_tmp_chg(void *data, Evas_Object *obj __UNUSED__, void *event __UNUSED__)
+_cb_op_tmp_chg(void *data, Evas_Object *obj EINA_UNUSED, void *event EINA_UNUSED)
 {
    Config *config = data;
    config->temporary = elm_check_state_get(obj);
 }
 
 static Eina_Bool
-_cb_op_del_delay(void *data __UNUSED__)
+_cb_op_del_delay(void *data EINA_UNUSED)
 {
    evas_object_del(op_opbox);
    evas_object_del(op_frame);
    options_font_clear();
+   options_wallpaper_clear();
    options_theme_clear();
    op_opbox = NULL;
    op_frame = NULL;
@@ -99,35 +70,52 @@ _cb_op_del_delay(void *data __UNUSED__)
 }
 
 static void
-_cb_mouse_down(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *ev __UNUSED__)
+_cb_mouse_down(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *ev EINA_UNUSED)
 {
-   options_toggle(saved_win, saved_bg, data);
+   options_toggle(saved_win, saved_bg, data, op_donecb, op_donedata);
 }
 
 static void
-_cb_opdt_hide_done(void *data, Evas_Object *obj __UNUSED__, const char *sig __UNUSED__, const char *src __UNUSED__)
+_cb_opdt_hide_done(void *data, Evas_Object *obj EINA_UNUSED, const char *sig EINA_UNUSED, const char *src EINA_UNUSED)
 {
    elm_box_clear(op_opbox);
-   switch (mode)
+   switch (_mode)
      {
-      case 1: options_font(op_opbox, data); break;
-      case 2: options_theme(op_opbox, data); break;
-      case 3: options_wallpaper(op_opbox, data); break;
-      case 4: options_colors(op_opbox, data); break;
-      case 5: options_video(op_opbox, data); break;
-      case 6: options_behavior(op_opbox, data); break;
-      case 7: options_helpers(op_opbox, data); break;
-      default: break;
+      case OPTION_NONE:      break;
+      case OPTION_FONT:      options_font(op_opbox, data); break;
+      case OPTION_THEME:     options_theme(op_opbox, data); break;
+      case OPTION_WALLPAPER: options_wallpaper(op_opbox, data); break;
+      case OPTION_COLORS:    options_colors(op_opbox, data); break;
+      case OPTION_VIDEO:     options_video(op_opbox, data); break;
+      case OPTION_BEHAVIOR:  options_behavior(op_opbox, data); break;
+      case OPTION_KEYS:      options_keys(op_opbox, data); break;
+      case OPTION_HELPERS:   options_helpers(op_opbox, data); break;
+      case OPTION_ELM:       options_elm(op_opbox, data); break;
      }
    edje_object_signal_emit(saved_bg, "optdetails,show", "terminology");
 }
 
+static void
+_cb_opdt_hide_done2(void *data EINA_UNUSED, Evas_Object *obj EINA_UNUSED, const char *sig EINA_UNUSED, const char *src EINA_UNUSED)
+{
+   if (op_del_timer)
+     {
+        ecore_timer_del(op_del_timer);
+        op_del_timer = NULL;
+     }
+   _cb_op_del_delay(NULL);
+   edje_object_signal_callback_del(saved_bg, "optdetails,hide,done",
+                                   "terminology",
+                                   _cb_opdt_hide_done2);
+}
+
 void
-options_toggle(Evas_Object *win, Evas_Object *bg, Evas_Object *term)
+options_toggle(Evas_Object *win, Evas_Object *bg, Evas_Object *term,
+               void (*donecb) (void *data), void *donedata)
 {
    Evas_Object *o;
 
-   mode = -1;
+   _mode = OPTION_NONE;
    if (!op_frame)
      {
         Elm_Object_Item *it_fn;
@@ -146,7 +134,7 @@ options_toggle(Evas_Object *win, Evas_Object *bg, Evas_Object *term)
         op_frame = o = elm_frame_add(win);
         evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
         evas_object_size_hint_align_set(o, EVAS_HINT_FILL, EVAS_HINT_FILL);
-        elm_object_text_set(o, "Options");
+        elm_object_text_set(o, _("Options"));
 
         op_box = o = elm_box_add(win);
         elm_box_horizontal_set(o, EINA_TRUE);
@@ -170,14 +158,21 @@ options_toggle(Evas_Object *win, Evas_Object *bg, Evas_Object *term)
         elm_toolbar_menu_parent_set(o, win);
         elm_toolbar_homogeneous_set(o, EINA_FALSE);
 
-        it_fn = 
-        elm_toolbar_item_append(o, "preferences-desktop-font",       "Font",      _cb_op_font,      term);
-        elm_toolbar_item_append(o, "preferences-desktop-theme",      "Theme",     _cb_op_theme,     term);
-        elm_toolbar_item_append(o, "preferences-desktop-wallpaper",  "Wallpaper", _cb_op_wallpaper, term);
-        elm_toolbar_item_append(o, "preferences-color",              "Colors",    _cb_op_colors,    term);
-        elm_toolbar_item_append(o, "preferences-desktop-multimedia", "Video",     _cb_op_video,     term);
-        elm_toolbar_item_append(o, "system-run",                     "Behavior",  _cb_op_behavior,  term);
-        elm_toolbar_item_append(o, "document-open",                  "Helpers",   _cb_op_helpers,   term);
+#define ITEM_APPEND(_icon_name, _name, _option_mode) \
+        elm_toolbar_item_append(o, _icon_name, _name, _cb_op, \
+                                (void*) OPTION_##_option_mode)
+
+        it_fn =
+        ITEM_APPEND("preferences-desktop-font", _("Font"), FONT);
+        ITEM_APPEND("preferences-desktop-theme", _("Theme"), THEME);
+        ITEM_APPEND("preferences-desktop-wallpaper", _("Wallpaper"), WALLPAPER);
+        ITEM_APPEND("video-display", _("Video"), VIDEO);
+        ITEM_APPEND("preferences-desktop-theme", _("Colors"), COLORS);
+        ITEM_APPEND("preferences-system", _("Behavior"), BEHAVIOR);
+        ITEM_APPEND("preferences-desktop-keyboard-shortcuts", _("Keys"), KEYS);
+        ITEM_APPEND("system-run", _("Helpers"), HELPERS);
+        ITEM_APPEND("apps", _("Toolkit"), ELM);
+#undef ITEM_APPEND
 
         elm_box_pack_end(op_tbox, o);
         evas_object_show(o);
@@ -187,7 +182,7 @@ options_toggle(Evas_Object *win, Evas_Object *bg, Evas_Object *term)
         op_temp = o = elm_check_add(win);
         evas_object_size_hint_weight_set(o, EVAS_HINT_EXPAND, 0.0);
         evas_object_size_hint_align_set(o, EVAS_HINT_FILL, 1.0);
-        elm_object_text_set(o, "Temporary");
+        elm_object_text_set(o, _("Temporary"));
         elm_check_state_set(o, config->temporary);
         elm_box_pack_end(op_tbox, o);
         evas_object_show(o);
@@ -218,6 +213,8 @@ options_toggle(Evas_Object *win, Evas_Object *bg, Evas_Object *term)
 
         edje_object_signal_emit(bg, "options,show", "terminology");
         op_out = EINA_TRUE;
+        op_donecb = donecb;
+        op_donedata = donedata;
         elm_object_focus_set(op_toolbar, EINA_TRUE);
         if (op_del_timer)
           {
@@ -230,13 +227,18 @@ options_toggle(Evas_Object *win, Evas_Object *bg, Evas_Object *term)
         edje_object_signal_callback_del(bg, "optdetails,hide,done",
                                         "terminology",
                                         _cb_opdt_hide_done);
+        edje_object_signal_callback_add(bg, "optdetails,hide,done",
+                                        "terminology",
+                                        _cb_opdt_hide_done2, term);
+        elm_object_focus_set(op_frame, EINA_FALSE);
+        elm_object_focus_set(op_opbox, EINA_FALSE);
+        elm_object_focus_set(op_toolbar, EINA_FALSE);
+        if (op_donecb) op_donecb(op_donedata);
         evas_object_del(op_over);
         op_over = NULL;
         edje_object_signal_emit(bg, "options,hide", "terminology");
         edje_object_signal_emit(bg, "optdetails,hide", "terminology");
         op_out = EINA_FALSE;
-        elm_object_focus_set(op_frame, EINA_FALSE);
-        elm_object_focus_set(term, EINA_TRUE);
         if (op_del_timer) ecore_timer_del(op_del_timer);
         op_del_timer = ecore_timer_add(10.0, _cb_op_del_delay, NULL);
      }
