@@ -6,8 +6,7 @@
 #include "sel.h"
 #include "config.h"
 #include "utils.h"
-#include "termio.h"
-#include "media.h"
+#include "term_container.h"
 
 typedef struct _Sel Sel;
 typedef struct _Entry Entry;
@@ -29,35 +28,41 @@ struct _Sel
    int w, h;
    struct {
       Evas_Coord x, y;
-      Eina_Bool down : 1;
+      unsigned char down : 1;
    } down;
-   Eina_Bool select_me : 1;
-   Eina_Bool exit_me : 1;
-   Eina_Bool exit_on_sel : 1;
-   Eina_Bool exit_now : 1;
+   Config *config;
+   unsigned char select_me : 1;
+   unsigned char exit_me : 1;
+   unsigned char exit_on_sel : 1;
+   unsigned char exit_now : 1;
+   unsigned char pending_sel : 1;
+   unsigned char use_px : 1;
 };
 
 struct _Entry
 {
-   Evas_Object *obj, *bg, *media, *termio;
-   Eina_Bool selected : 1;
-   Eina_Bool selected_before : 1;
-   Eina_Bool selected_orig : 1;
+   Evas_Object *obj, *bg;
+   Term_Container *tc;
+   unsigned char selected : 1;
+   unsigned char selected_before : 1;
+   unsigned char selected_orig : 1;
+   unsigned char was_selected : 1;
 };
 
 static Evas_Smart *_smart = NULL;
 static Evas_Smart_Class _parent_sc = EVAS_SMART_CLASS_INIT_NULL;
 
 static void _smart_calculate(Evas_Object *obj);
-
+static void _transit(Evas_Object *obj, double tim);
 
 static void
-_mouse_down_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event)
+_mouse_down_cb(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event)
 {
    Evas_Event_Mouse_Down *ev = event;
    Sel *sd = evas_object_smart_data_get(data);
    if (!sd) return;
-   
+
+   if (sd->exit_on_sel) return;
    if (sd->down.down) return;
    if (ev->button != 1) return;
    sd->down.x = ev->canvas.x;
@@ -66,20 +71,81 @@ _mouse_down_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void
 }
 
 static void
-_mouse_up_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event)
+_mouse_up_cb(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event)
 {
-   Evas_Event_Mouse_Down *ev = event;
+   Evas_Event_Mouse_Up *ev = event;
    Sel *sd = evas_object_smart_data_get(data);
    Evas_Coord dx, dy;
    if (!sd) return;
-   
+
+   if (sd->exit_on_sel) return;
    if (!sd->down.down) return;
    sd->down.down = EINA_FALSE;
    dx = abs(ev->canvas.x - sd->down.x);
    dy = abs(ev->canvas.y - sd->down.y);
    if ((dx <= elm_config_finger_size_get()) &&
        (dy <= elm_config_finger_size_get()))
-     evas_object_smart_callback_call(data, "clicked", NULL);
+     {
+        Eina_List *l;
+        Entry *en;
+        Evas_Coord x, y, ox, oy, ow, oh;
+
+        x = ev->canvas.x;
+        y = ev->canvas.y;
+        EINA_LIST_FOREACH(sd->items, l, en)
+          {
+             evas_object_geometry_get(en->bg, &ox, &oy, &ow, &oh);
+             if (ELM_RECTS_INTERSECT(ox, oy, ow, oh, x, y, 1, 1))
+               {
+                  sel_entry_selected_set(data, en->obj, EINA_FALSE);
+                  sd->select_me = EINA_TRUE;
+                  sd->exit_me = EINA_FALSE;
+                  if (sd->autozoom_timeout)
+                    {
+                       ecore_timer_del(sd->autozoom_timeout);
+                       sd->autozoom_timeout = NULL;
+                    }
+                  evas_object_smart_callback_call(data, "ending", NULL);
+                  sel_zoom(data, 1.0);
+                  return;
+               }
+          }
+        evas_object_smart_callback_call(data, "clicked", NULL);
+     }
+}
+
+static void
+_mouse_move_cb(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *event)
+{
+   Evas_Event_Mouse_Move *ev = event;
+   Sel *sd = evas_object_smart_data_get(data);
+   if (!sd) return;
+   Evas_Coord x = 0, y = 0, w = 0, h = 0, sw, sh;
+   int iw, ih;
+
+   if ((sd->exit_me) || (sd->exit_now) || (sd->select_me) || (sd->exit_on_sel))
+     return;
+   iw = sqrt(eina_list_count(sd->items));
+   if (iw < 1) iw = 1;
+   ih = (eina_list_count(sd->items) + (iw - 1)) / iw;
+   if (ih < 1) ih = 1;
+   evas_object_geometry_get(data, &x, &y, &w, &h);
+   sw = w * sd->zoom;
+   sh = h * sd->zoom;
+   if (!sd->down.down)
+     {
+        if ((w > 0) && (h > 0))
+          {
+             _transit(data, 0.5);
+             sd->px1 = ((ev->cur.canvas.x - x) * ((iw - 1) * sw)) / w;
+             sd->py1 = ((ev->cur.canvas.y - y) * ((ih - 1) * sh)) / h;
+             sd->use_px = EINA_TRUE;
+          }
+     }
+   else
+     {
+        // XXX: drag to scroll
+     }
 }
 
 static Eina_Bool
@@ -108,15 +174,16 @@ _autozoom(Evas_Object *obj)
 }
 
 void
-_key_down_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *event)
+_key_down_cb(void *data, Evas *e EINA_UNUSED, Evas_Object *obj, void *event)
 {
    Evas_Event_Key_Down *ev = event;
    Sel *sd = evas_object_smart_data_get(data);
    Eina_List *l;
    Entry *en;
+
    if (!sd) return;
-   if ((!strcmp(ev->keyname, "Next")) ||
-       (!strcmp(ev->keyname, "Right")))
+   if ((!strcmp(ev->key, "Next")) ||
+       (!strcmp(ev->key, "Right")))
      {
         EINA_LIST_FOREACH(sd->items, l, en)
           {
@@ -128,13 +195,14 @@ _key_down_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *
                        sel_entry_selected_set(obj, en->obj, EINA_FALSE);
                        break;
                     }
+                  else return;
                }
           }
         sd->exit_now = EINA_FALSE;
         _autozoom(data);
      }
-   else if ((!strcmp(ev->keyname, "Prior")) ||
-            (!strcmp(ev->keyname, "Left")))
+   else if ((!strcmp(ev->key, "Prior")) ||
+            (!strcmp(ev->key, "Left")))
      {
         EINA_LIST_FOREACH(sd->items, l, en)
           {
@@ -146,14 +214,75 @@ _key_down_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *
                        sel_entry_selected_set(obj, en->obj, EINA_FALSE);
                        break;
                     }
+                  else return;
                }
           }
         sd->exit_now = EINA_FALSE;
         _autozoom(data);
      }
-   else if ((!strcmp(ev->keyname, "Return")) ||
-            (!strcmp(ev->keyname, "KP_Enter")) ||
-            (!strcmp(ev->keyname, "space")))
+   else if (!strcmp(ev->key, "Up"))
+     {
+        EINA_LIST_FOREACH(sd->items, l, en)
+          {
+             if (en->selected)
+               {
+                  Evas_Coord x = 0, y = 0, w = 0, h = 0, sgx, sgy;
+                  Eina_Bool found = EINA_FALSE;
+
+                  evas_object_geometry_get(en->bg, &x, &y, &w, &h);
+                  sgx = x + (w / 2);
+                  sgy = y - (h / 2);
+                  EINA_LIST_FOREACH(sd->items, l, en)
+                    {
+                       evas_object_geometry_get(en->bg, &x, &y, &w, &h);
+                       if (ELM_RECTS_INTERSECT(x, y, w, h, sgx, sgy, 1, 1))
+                         {
+                            sel_entry_selected_set(obj, en->obj, EINA_FALSE);
+                            found = EINA_TRUE;
+                            break;
+                         }
+                    }
+                  if (found == EINA_FALSE)
+                    return;
+                  break;
+               }
+          }
+        sd->exit_now = EINA_FALSE;
+        _autozoom(data);
+     }
+   else if (!strcmp(ev->key, "Down"))
+     {
+        EINA_LIST_FOREACH(sd->items, l, en)
+          {
+             if (en->selected)
+               {
+                  Evas_Coord x = 0, y = 0, w = 0, h = 0, sgx, sgy;
+                  Eina_Bool found = EINA_FALSE;
+
+                  evas_object_geometry_get(en->bg, &x, &y, &w, &h);
+                  sgx = x + (w / 2);
+                  sgy = y + h + (h / 2);
+                  EINA_LIST_FOREACH(sd->items, l, en)
+                    {
+                       evas_object_geometry_get(en->bg, &x, &y, &w, &h);
+                       if (ELM_RECTS_INTERSECT(x, y, w, h, sgx, sgy, 1, 1))
+                         {
+                            sel_entry_selected_set(obj, en->obj, EINA_FALSE);
+                            found = EINA_TRUE;
+                            break;
+                         }
+                    }
+                  if (found == EINA_FALSE)
+                    return;
+                  break;
+               }
+          }
+        sd->exit_now = EINA_FALSE;
+        _autozoom(data);
+     }
+   else if ((!strcmp(ev->key, "Return")) ||
+            (!strcmp(ev->key, "KP_Enter")) ||
+            (!strcmp(ev->key, "space")))
      {
         sd->select_me = EINA_TRUE;
         sd->exit_me = EINA_FALSE;
@@ -165,10 +294,10 @@ _key_down_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *
         evas_object_smart_callback_call(data, "ending", NULL);
         sel_zoom(data, 1.0);
      }
-   else if (!strcmp(ev->keyname, "Escape"))
+   else if (!strcmp(ev->key, "Escape"))
      {
         Evas_Object *entry = NULL;
-        
+
         EINA_LIST_FOREACH(sd->items, l, en)
           {
              if (en->selected_orig) entry = en->obj;
@@ -203,32 +332,46 @@ _layout(Evas_Object *obj)
    w = ow * sd->zoom;
    h = oh * sd->zoom;
    x = y = 0;
-   EINA_LIST_FOREACH(sd->items, l, en)
+   if (!sd->use_px)
      {
-        if (en->selected_before)
+        EINA_LIST_FOREACH(sd->items, l, en)
           {
-             sd->px0 = (x * w);
-             sd->py0 = (y * h);
-          }
-        if (en->selected)
-          {
-             sd->px1 = (x * w);
-             sd->py1 = (y * h);
-          }
-        x++;
-        if (x >= iw)
-          {
-             x = 0;
-             y++;
+             if (en->selected_before)
+               {
+                  sd->px0 = (x * w);
+                  sd->py0 = (y * h);
+               }
+             if ((sd->exit_on_sel) && (!sd->exit_now))
+               {
+                  if (en->selected_before)
+                    {
+                       sd->px1 = (x * w);
+                       sd->py1 = (y * h);
+                    }
+               }
+             else
+               {
+                  if (en->selected)
+                    {
+                       sd->px1 = (x * w);
+                       sd->py1 = (y * h);
+                    }
+               }
+             x++;
+             if (x >= iw)
+               {
+                  x = 0;
+                  y++;
+               }
           }
      }
 
    sd->px = sd->px0 + ((sd->px1 - sd->px0) * sd->interp);
    sd->py = sd->py0 + ((sd->py1 - sd->py0) * sd->interp);
-   
+
    px = sd->px;
    py = sd->py;
-   
+
    if ((iw > 0) && (w > 0) && (ih > 0) && (h > 0))
      {
         ww = iw * w;
@@ -239,11 +382,11 @@ _layout(Evas_Object *obj)
         else py = py - ((py * (oh - h)) / (hh - h));
      }
    x = y = 0;
+
    EINA_LIST_FOREACH(sd->items, l, en)
      {
         evas_object_move(en->bg, ox + (x * w) - px, oy + (y * h) - py);
         evas_object_resize(en->bg, w, h);
-        evas_object_show(en->obj);
         evas_object_show(en->bg);
         x++;
         if (x >= iw)
@@ -252,6 +395,17 @@ _layout(Evas_Object *obj)
              y++;
           }
      }
+   if ((sd->w > 0) && (sd->h > 0) && (sd->pending_sel))
+     {
+        sd->pending_sel = EINA_FALSE;
+        EINA_LIST_FOREACH(sd->items, l, en)
+          {
+             if ((!en->was_selected) && (en->selected))
+               edje_object_signal_emit(en->bg, "selected", "terminology");
+             else if ((en->was_selected) && (!en->selected))
+               edje_object_signal_emit(en->bg, "unselected", "terminology");
+          }
+     }
 }
 
 static Eina_Bool
@@ -290,7 +444,7 @@ _anim_cb(void *data)
              Eina_List *l;
              Entry *en;
              Evas_Object *entry = NULL;
-             
+
              EINA_LIST_FOREACH(sd->items, l, en)
                {
                   if (en->selected) entry = en->obj;
@@ -322,60 +476,53 @@ static void
 _label_redo(Entry *en)
 {
    const char *s;
-   
-   if (!en->obj) return;
-   if (!en->termio) return;
-   s = termio_title_get(en->termio);
-   if (!s) s = termio_icon_name_get(en->termio);
-   if (s) edje_object_part_text_set(en->bg, "terminology.label", s);
+
+   if (!en->obj || !en->tc)
+     return;
+   s = en->tc->title;
+   if (!s)
+     s = "Terminology";
+   edje_object_part_text_set(en->bg, "terminology.label", s);
 }
 
-static void
-_title_cb(void *data, Evas_Object *obj __UNUSED__, void *info __UNUSED__)
+void
+sel_entry_title_set(void *entry, const char *title)
 {
-   _label_redo(data);
+   Entry *en = entry;
+
+   edje_object_part_text_set(en->bg, "terminology.label", title);
 }
 
-static void
-_icon_cb(void *data, Evas_Object *obj __UNUSED__, void *info __UNUSED__)
-{
-   _label_redo(data);
-}
-
-static void
-_bell_cb(void *data, Evas_Object *obj __UNUSED__, void *info __UNUSED__)
+void
+sel_entry_close(void *data)
 {
    Entry *en = data;
-   edje_object_signal_emit(en->bg, "bell", "terminology");
+
+   en->tc = NULL;
 }
 
-static void
-_media_del_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *info __UNUSED__)
+void
+sel_entry_update(void *data)
 {
    Entry *en = data;
-   en->media = NULL;
+
+   en->tc = evas_object_data_get(en->obj, "tc");
+   if (en->tc)
+     {
+        _label_redo(en);
+     }
 }
 
 static void
-_entry_termio_del_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *info __UNUSED__)
-{
-   Entry *en = data;
-   if (en->termio) evas_object_event_callback_add
-     (en->termio, EVAS_CALLBACK_DEL, _entry_termio_del_cb, en);
-   en->termio = NULL;
-}
-
-static void
-_entry_del_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void *info __UNUSED__)
+_entry_del_cb(void *data, Evas *e EINA_UNUSED, Evas_Object *obj EINA_UNUSED, void *info EINA_UNUSED)
 {
    Entry *en = data;
    if (en->obj) evas_object_event_callback_del_full
      (en->obj, EVAS_CALLBACK_DEL, _entry_del_cb, en);
    en->obj = NULL;
+   /*
    if (en->termio)
      {
-        evas_object_event_callback_del_full(en->termio, EVAS_CALLBACK_DEL,
-                                            _entry_termio_del_cb, en);
         evas_object_smart_callback_del_full(en->termio, "title,change",
                                             _title_cb, en);
         evas_object_smart_callback_del_full(en->termio, "icon,change",
@@ -384,6 +531,7 @@ _entry_del_cb(void *data, Evas *e __UNUSED__, Evas_Object *obj __UNUSED__, void
                                             _bell_cb, en);
         en->termio = NULL;
      }
+   */
 }
 
 static void
@@ -416,21 +564,9 @@ _smart_del(Evas_Object *obj)
    if (sd->autozoom_timeout) ecore_timer_del(sd->autozoom_timeout);
    EINA_LIST_FREE(sd->items, en)
      {
-        if (en->termio)
-          {
-             evas_object_event_callback_del_full(en->termio, EVAS_CALLBACK_DEL,
-                                                 _entry_termio_del_cb, en);
-             evas_object_smart_callback_del_full(en->termio, "title,change",
-                                                 _title_cb, en);
-             evas_object_smart_callback_del_full(en->termio, "icon,change",
-                                                 _icon_cb, en);
-             evas_object_smart_callback_del_full(en->termio, "bell",
-                                                 _bell_cb, en);
-          }
         if (en->obj) evas_object_event_callback_del_full
           (en->obj, EVAS_CALLBACK_DEL, _entry_del_cb, en);
         if (en->obj) evas_object_del(en->obj);
-        if (en->media) evas_object_del(en->media);
         evas_object_del(en->bg);
         free(en);
      }
@@ -441,12 +577,11 @@ static void
 _smart_resize(Evas_Object *obj, Evas_Coord w, Evas_Coord h)
 {
    Sel *sd = evas_object_smart_data_get(obj);
-   Evas_Coord ox, oy, ow, oh;
+   Evas_Coord ow, oh;
    if (!sd) return;
-   evas_object_geometry_get(obj, &ox, &oy, &ow, &oh);
+   evas_object_geometry_get(obj, NULL, NULL, &ow, &oh);
    if ((ow == w) && (oh == h)) return;
    evas_object_smart_changed(obj);
-   evas_object_resize(sd->clip, ow, oh);
 }
 
 static void
@@ -461,11 +596,13 @@ _smart_calculate(Evas_Object *obj)
    sd->h = oh;
    evas_object_move(sd->clip, ox, oy);
    evas_object_resize(sd->clip, ow, oh);
+   evas_object_move(sd->o_event, ox, oy);
+   evas_object_resize(sd->o_event, ow, oh);
    _layout(obj);
 }
 
 static void
-_smart_move(Evas_Object *obj, Evas_Coord x __UNUSED__, Evas_Coord y __UNUSED__)
+_smart_move(Evas_Object *obj, Evas_Coord x EINA_UNUSED, Evas_Coord y EINA_UNUSED)
 {
    Sel *sd = evas_object_smart_data_get(obj);
    if (!sd) return;
@@ -478,7 +615,7 @@ _smart_init(void)
 
    evas_object_smart_clipped_smart_set(&_parent_sc);
    sc           = _parent_sc;
-   sc.name      = "media";
+   sc.name      = "sel";
    sc.version   = EVAS_SMART_CLASS_VERSION;
    sc.add       = _smart_add;
    sc.del       = _smart_del;
@@ -503,7 +640,7 @@ sel_add(Evas_Object *parent)
    obj = evas_object_smart_add(e, _smart);
    sd = evas_object_smart_data_get(obj);
    if (!sd) return obj;
-   
+
    sd->o_event = evas_object_rectangle_add(e);
    evas_object_color_set(sd->o_event, 0, 0, 0, 0);
    evas_object_repeat_events_set(sd->o_event, EINA_TRUE);
@@ -514,20 +651,24 @@ sel_add(Evas_Object *parent)
                                   _mouse_down_cb, obj);
    evas_object_event_callback_add(sd->o_event, EVAS_CALLBACK_MOUSE_UP,
                                   _mouse_up_cb, obj);
+   evas_object_event_callback_add(sd->o_event, EVAS_CALLBACK_MOUSE_MOVE,
+                                  _mouse_move_cb, obj);
    evas_object_event_callback_add(obj, EVAS_CALLBACK_KEY_DOWN,
                                   _key_down_cb, obj);
    sd->zoom = 1.0;
-   
+
    return obj;
 }
 
-void
-sel_entry_add(Evas_Object *obj, Evas_Object *entry, Eina_Bool selected, Config *config)
+void *
+sel_entry_add(Evas_Object *obj, Evas_Object *entry,
+              Eina_Bool selected, Eina_Bool bell, Config *config)
 {
    Sel *sd = evas_object_smart_data_get(obj);
    Entry *en = calloc(1, sizeof(Entry));
-   if (!en) return;
+   if (!en) return NULL;
    sd->items = eina_list_append(sd->items, en);
+   sd->config = config;
    en->obj = entry;
    en->selected = selected;
    en->selected_before = selected;
@@ -539,47 +680,30 @@ sel_entry_add(Evas_Object *obj, Evas_Object *entry, Eina_Bool selected, Config *
    edje_object_part_swallow(en->bg, "terminology.content", en->obj);
    evas_object_show(en->obj);
 
-   if (config->background)
-     {
-        Evas_Object *o;
-        int type = 0;
-        
-        en->media = o = media_add(obj,
-                                  config->background, config,
-                                  MEDIA_BG, &type);
-        evas_object_event_callback_add(o, EVAS_CALLBACK_DEL,
-                                       _media_del_cb, en);
-        edje_object_part_swallow(en->bg, "terminology.background", o);
-        evas_object_show(o);
-        if (type == TYPE_IMG)
-          edje_object_signal_emit(en->bg, "media,image", "terminology");
-        else if (type == TYPE_SCALE)
-          edje_object_signal_emit(en->bg, "media,scale", "terminology");
-        else if (type == TYPE_EDJE)
-          edje_object_signal_emit(en->bg, "media,edje", "terminology");
-        else if (type == TYPE_MOV)
-          edje_object_signal_emit(en->bg, "media,movie", "terminology");
-     }
-   
    evas_object_stack_below(en->bg, sd->o_event);
    if (en->selected)
-     edje_object_signal_emit(en->bg, "selected,start", "terminology");
-   sd->interp = 1.0;
-   en->termio = evas_object_data_get(en->obj, "termio");
-   if (en->termio)
      {
-        evas_object_smart_callback_add(en->termio, "title,change",
-                                       _title_cb, en);
-        evas_object_smart_callback_add(en->termio, "icon,change",
-                                       _icon_cb, en);
-        evas_object_smart_callback_add(en->termio, "bell",
-                                       _bell_cb, en);
+        edje_object_signal_emit(en->bg, "selected,start", "terminology");
+        edje_object_message_signal_process(en->bg);
+     }
+   if (bell)
+     {
+        edje_object_signal_emit(en->bg, "bell", "terminology");
+        if (!config->bell_rings)
+          edje_object_signal_emit(en->bg, "bell,ring", "terminology");
+
+        edje_object_message_signal_process(en->bg);
+     }
+   sd->interp = 1.0;
+   en->tc = evas_object_data_get(en->obj, "tc");
+   if (en->tc)
+     {
         _label_redo(en);
-        evas_object_event_callback_add(en->termio, EVAS_CALLBACK_DEL,
-                                       _entry_termio_del_cb, en);
      }
    evas_object_event_callback_add(en->obj, EVAS_CALLBACK_DEL,
                                   _entry_del_cb, en);
+
+   return en;
 }
 
 void
@@ -607,26 +731,42 @@ sel_entry_selected_set(Evas_Object *obj, Evas_Object *entry, Eina_Bool keep_befo
    Sel *sd = evas_object_smart_data_get(obj);
    Eina_List *l;
    Entry *en;
+   Config *config;
    if (!sd) return;
+
+   config = sd->config;
+   if (!config) return;
+
    EINA_LIST_FOREACH(sd->items, l, en)
      {
         if (en->obj == entry)
           {
-             edje_object_signal_emit(en->bg, "selected", "terminology");
+             if ((sd->w > 0) && (sd->h > 0))
+               edje_object_signal_emit(en->bg, "selected", "terminology");
+             else
+               sd->pending_sel = EINA_TRUE;
              evas_object_stack_below(en->bg, sd->o_event);
+             en->was_selected = EINA_FALSE;
              en->selected = EINA_TRUE;
           }
         else if (en->obj != entry)
           {
              if (en->selected)
                {
-                  edje_object_signal_emit(en->bg, "unselected", "terminology");
+                  if ((sd->w > 0) && (sd->h > 0))
+                    edje_object_signal_emit(en->bg, "unselected", "terminology");
+                  else
+                    {
+                       en->was_selected = EINA_TRUE;
+                       sd->pending_sel = EINA_TRUE;
+                    }
                   en->selected = EINA_FALSE;
                }
           }
         if (!keep_before) en->selected_before = EINA_FALSE;
      }
-   _transit(obj, 0.5);
+   sd->use_px = EINA_FALSE;
+   _transit(obj, config->tab_zoom);
 }
 
 void
@@ -634,8 +774,11 @@ sel_zoom(Evas_Object *obj, double zoom)
 {
    Sel *sd = evas_object_smart_data_get(obj);
    if (!sd) return;
+   Config *config = sd->config;
+   if (!config) return;
+
    sd->zoom1 = zoom;
-   _transit(obj, 0.5);
+   _transit(obj, config->tab_zoom);
 }
 
 void
