@@ -1,2 +1,6 @@
+#ifndef _TERMPTY_EXT_H__
+#define _TERMPTY_EXT_H__ 1
+
 Eina_Bool _termpty_ext_handle(Termpty *ty, const char *txt, Eina_Unicode *utxt);
-    
+
+#endif
