@@ -8,11 +8,11 @@ int
 main(int argc, char **argv)
 {
    int i, perm = 0;
-   
+
    if (!getenv("TERMINOLOGY")) return 0;
    if (argc <= 1)
      {
-        printf("Usage: %s [-p] on|off\n"
+        printf("Usage: %s [-p] on|off|<opacity level>\n"
                "  Change the terminal transparency on or off\n"
                "  -p  Make change permanent (stored in config)\n"
                "\n",
@@ -22,7 +22,7 @@ main(int argc, char **argv)
    for (i = 1; i < argc; i++)
      {
         char tbuf[PATH_MAX];
-        
+
         if (!strcmp(argv[i], "-p"))
           {
              perm = 1;
@@ -33,10 +33,7 @@ main(int argc, char **argv)
           snprintf(tbuf, sizeof(tbuf), "%c}ap%s", 0x1b, argv[i]);
         else
           snprintf(tbuf, sizeof(tbuf), "%c}at%s", 0x1b, argv[i]);
-        if (write(0, tbuf, strlen(tbuf)) < 1) perror("write");
-        tbuf[0] = 0;
-        if (write(0, tbuf, 1) < 1) perror("write");
+        if (write(0, tbuf, strlen(tbuf) + 1) != (signed)(strlen(tbuf) + 1)) perror("write");
      }
-   exit(0);
    return 0;
 }
