@@ -25,9 +25,7 @@ Ecore_Evas *ee = NULL;
 Evas *evas = NULL;
 Evas_Object *o = NULL;
 struct termios told, tnew;
-int tw = 0, th = 0, cw = 0, ch = 0;
-
-#include "extns.h"
+int tw = 0, th = 0;
 
 static int
 echo_off(void)
@@ -46,72 +44,20 @@ echo_on(void)
 }
 
 static void
-scaleterm(int w, int h, int *iw, int *ih)
+size_print(char *buf, int bufsz, char *sz, long long size)
 {
-   if (w > (tw * cw))
-     {
-        *iw = tw;
-        *ih = ((h * (tw * cw) / w) + (ch - 1)) / ch;
-     }
-   else
-     {
-        *iw = (w + (cw - 1)) / cw;
-        *ih = (h + (ch - 1)) / ch;
-     }
-}
+   char *prefix = " KMGTPEZY";
 
-static const char *
-is_fmt(const char *f, const char **extn)
-{
-   int i, len, l;
-   
-   len = strlen(f);
-   for (i = 0; extn[i]; i++)
+   while (*prefix)
      {
-        l = strlen(extn[i]);
-        if (len < l) continue;
-        if (!strcasecmp(extn[i], f + len - l)) return extn[i];
-     }
-   return NULL;
-}
-
-static void
-size_print(char *buf, int bufsz, char *sz, unsigned long long size)
-{
-   if (size < 1024LL)
-     {
-        snprintf(buf, bufsz, "%4lld", size);
-        *sz = ' ';
-     }
-   else if (size < (1024LL * 1024LL))
-     {
-        snprintf(buf, bufsz, "%4lld", size / (1024LL));
-        *sz = 'K';
-     }
-   else if (size < (1024LL * 1024LL * 1024LL))
-     {
-        snprintf(buf, bufsz, "%4lld", size / (1024LL * 1024LL));
-        *sz = 'M';
-     }
-   else if (size < (1024LL * 1024LL * 1024LL * 1024LL))
-     {
-        snprintf(buf, bufsz, "%4lld", size / (1024LL * 1024 * 1024LL));
-        *sz = 'G';
-     }
-   else if (size < (1024LL * 1024LL * 1024LL * 1024LL * 1024LL))
-     {
-        snprintf(buf, bufsz, "%4lld", size / (1024LL * 1024LL * 1024LL * 1024LL));
-        *sz = 'T';
-     }
-   else if (size < (1024LL * 1024LL * 1024LL * 1024LL * 1024LL * 1024LL))
-     {
-        snprintf(buf, bufsz, "%4lld", size / (1024LL * 1024LL * 1024LL * 1024LL * 1024LL));
-        *sz = 'P';
-     }
-   else
-     {
-        snprintf(buf, bufsz, "%4lld", size / (1024LL * 1024LL * 1024LL * 1024LL * 1024LL * 1024LL));
-        *sz = 'E';
+        if ((size < 1024) || (*prefix == 'Y'))
+          {
+             snprintf(buf, bufsz, "%4lld", size);
+             *sz = *prefix;
+             return;
+          }
+        size >>= 10;
+        prefix++;
      }
 }
 
@@ -133,6 +79,8 @@ size_print(char *buf, int bufsz, char *sz, unsigned long long size)
 #define WHITE   7
 #define BRIGHT  8
 
+#define COUNT_OF(arr) (sizeof(arr) / sizeof(*arr))
+
 // #3399ff
 // 51 153 355
 // as 6x6x6
@@ -197,6 +145,12 @@ typedef struct _Cmatch
    const char *match, *icon;
 } Cmatch;
 
+typedef struct _Tyls_Options
+{
+  int mode;
+  Eina_Bool hidden;
+} Tyls_Options;
+
 // for regular files
 const Cmatch fmatch[] =
 {
@@ -234,6 +188,8 @@ const Cmatch fmatch[] =
    { 5, 5, 5,  9, 9, 9, "*.ICO", NULL},
    { 5, 5, 5,  9, 9, 9, "*.pgm", NULL},
    { 5, 5, 5,  9, 9, 9, "*.PGM", NULL},
+   { 5, 5, 5,  9, 9, 9, "*.tga", NULL},
+   { 5, 5, 5,  9, 9, 9, "*.TGA", NULL},
    { 5, 5, 5,  9, 9, 9, "*.pbm", NULL},
    { 5, 5, 5,  9, 9, 9, "*.PBM", NULL},
    { 5, 5, 5,  9, 9, 9, "*.psd", NULL},
@@ -345,12 +301,12 @@ const Cmatch fmatch[] =
    { 3, 4, 5,  9, 9, 9, "*.264", NULL},
    { 3, 4, 5,  9, 9, 9, "*.3gp", NULL},
    
-   { 2, 3, 5,  9, 9, 9, "*.mp3", NULL},
-   { 2, 3, 5,  9, 9, 9, "*.MP3", NULL},
-   { 2, 3, 5,  9, 9, 9, "*.aac", NULL},
-   { 2, 3, 5,  9, 9, 9, "*.AAC", NULL},
-   { 2, 3, 5,  9, 9, 9, "*.wav", NULL},
-   { 2, 3, 5,  9, 9, 9, "*.WAV", NULL},
+   { 2, 3, 5,  9, 9, 9, "*.mp3", "audio-x-mpeg.svg"},
+   { 2, 3, 5,  9, 9, 9, "*.MP3", "audio-x-mpeg.svg"},
+   { 2, 3, 5,  9, 9, 9, "*.aac", "audio-x-generic.svg"},
+   { 2, 3, 5,  9, 9, 9, "*.AAC", "audio-x-generic.svg"},
+   { 2, 3, 5,  9, 9, 9, "*.wav", "audio-x-wav.svg"},
+   { 2, 3, 5,  9, 9, 9, "*.WAV", "audio-x-wav.svg"},
    { 2, 3, 5,  9, 9, 9, "*.m3u", "audio-x-mp3-playlist"},
    { 2, 3, 5,  9, 9, 9, "*.M3U", "audio-x-mp3-playlist"},
    
@@ -527,16 +483,10 @@ fileicon(const char *path)
    name = ecore_file_file_get(path);
    if (name)
      {
-        char *ts;
-             
         if (ecore_file_is_dir(path)) isdir = EINA_TRUE;
-        ts = ecore_file_readlink(path);
-        if (ts)
-          {
-             free(ts);
-          }
+        free(ecore_file_readlink(path));
         if (ecore_file_can_exec(path)) isexec = EINA_TRUE;
-        
+
         if (isdir) return filematch(name, dmatch);
         else if (isexec) return filematch(name, xmatch);
         else return filematch(name, fmatch);
@@ -632,7 +582,7 @@ fileprint(char *path, char *name, Eina_Bool type)
 }
 
 static void
-list_dir(const char *dir, int mode)
+list_dir(const char *dir, Tyls_Options *options)
 {
    Eina_List *files, *l;
    char *s, **names;
@@ -647,15 +597,15 @@ list_dir(const char *dir, int mode)
      {
         int len = eina_unicode_utf8_get_len(s);
         
-        if (s[0] == '.') continue;
+        if (s[0] == '.' && options->hidden == EINA_FALSE) continue;
         if (len > maxlen) maxlen = len;
         names[i] = s;
         i++;
      }
    num = i;
    stuff = 0;
-   if (mode == SMALL) stuff += 2;
-   else if (mode == MEDIUM) stuff += 4;
+   if (options->mode == SMALL) stuff += 2;
+   else if (options->mode == MEDIUM) stuff += 4;
    stuff += 5; // xxxx[ /K/M/G/T/P...]
    stuff += 1; // spacer at start
    // name
@@ -672,13 +622,14 @@ list_dir(const char *dir, int mode)
              stuff--;
           }
         if (cols > num) cols = num;
+        if (cols == 0) cols = 1;
         rows = ((num + (cols - 1)) / cols);
         for (i = 0; i < rows; i++)
           {
              char buf[4096];
              const char *icon;
              
-             if (mode == SMALL)
+             if (options->mode == SMALL)
                {
                   for (c = 0; c < cols; c++)
                     {
@@ -708,7 +659,7 @@ list_dir(const char *dir, int mode)
                     }
                   printf("\n");
                }
-             else if (mode == MEDIUM)
+             else if (options->mode == MEDIUM)
                {
                   for (c = 0; c < cols; c++)
                     {
@@ -771,20 +722,26 @@ int
 main(int argc, char **argv)
 {
    char buf[64];
-   Eina_Bool listed = EINA_FALSE;
+   char *path;
+   Eina_List *dirs = NULL;
+   Tyls_Options options = {SMALL, EINA_FALSE};
    
    if (!getenv("TERMINOLOGY")) return 0;
    if ((argc == 2) && (!strcmp(argv[1], "-h")))
      {
-        printf("Usage: %s [-s|-m|-l] FILE1 [FILE2 ...]\n"
+        printf("Usage: %s [-a] [-s|-m] FILE1 [FILE2 ...]\n"
                "\n"
+               "  -a  Show hidden files\n"
                "  -s  Small list mode\n"
-               "  -m  Medium list mode\n"
-               "  -l  Large list mode\n",
+               "  -m  Medium list mode\n",
+               /*"  -l  Large list mode\n", Enable again once we support it */
               argv[0]);
         return 0;
      }
    eina_init();
+#if (ECORE_VERSION_MAJOR > 1) || (ECORE_VERSION_MINOR >= 8)
+   ecore_app_no_system_modules();
+#endif
    ecore_init();
    ecore_file_init();
    evas_init();
@@ -794,19 +751,17 @@ main(int argc, char **argv)
    ee = ecore_evas_buffer_new(1, 1);
    if (ee)
      {
-        int i, mode = SMALL;
+        int i, cw, ch;
+        int len;
         char *rp;
         
         evas = ecore_evas_get(ee);
         echo_off();
         snprintf(buf, sizeof(buf), "%c}qs", 0x1b);
-        if (write(0, buf, strlen(buf) + 1) < 0) perror("write");
-        if (scanf("%i;%i;%i;%i", &tw, &th, &cw, &ch) != 4)
-          {
-             echo_on();
-             return 0;
-          }
-        if ((tw <= 0) || (th <= 0) || (cw <= 1) || (ch <= 1))
+        len = strlen(buf);
+        if (write(0, buf, len + 1) < (signed)len + 1) perror("write");
+        if ((scanf("%i;%i;%i;%i", &tw, &th, &cw, &ch) != 4)
+            || (tw <= 0) || (th <= 0) || (cw <= 1) || (ch <= 1))
           {
              echo_on();
              return 0;
@@ -814,49 +769,40 @@ main(int argc, char **argv)
         echo_on();
         for (i = 1; i < argc; i++)
           {
-             char *path;
+             char *cmp[] = {"-s", "-m", "-l"};
+             int modes[] = {SMALL, MEDIUM, LARGE};
+             unsigned int j;
 
-             if (!strcmp(argv[i], "-c"))
+             for (j = 0; j < COUNT_OF(cmp) ; j++)
                {
-                  mode = SMALL;
-                  i++;
-                  if (i >= argc) break;
+                 if (!strcmp(argv[i], cmp[j]))
+                   {
+                      options.mode = modes[j];
+                   }
                }
-             else if (!strcmp(argv[i], "-m"))
+             if (!strcmp(argv[i], "-a"))
                {
-                  mode = MEDIUM;
-                  i++;
-                  if (i >= argc) break;
+                  options.hidden = EINA_TRUE;
                }
-             else if (!strcmp(argv[i], "-l"))
+             if (argv[i][0] != '-')
                {
-                  mode = LARGE;
-                  i++;
-                  if (i >= argc) break;;
-               }
-             path = argv[i];
-             rp = ecore_file_realpath(path);
-             if (rp)
-               {
-                  if (ecore_file_is_dir(rp))
-                    {
-                       list_dir(rp, mode);
-                       listed = EINA_TRUE;
-                    }
-                  free(rp);
+                  dirs = eina_list_append(dirs, argv[i]);
                }
           }
-        if (!listed)
+        if (!eina_list_count(dirs))
           {
-             rp = ecore_file_realpath("./");
-             if (rp)
+             dirs = eina_list_append(dirs, "./");
+          }
+        EINA_LIST_FREE(dirs, path)
+          {
+            if ((rp = ecore_file_realpath(path))
+                 && ecore_file_is_dir(rp))
                {
-                  list_dir(rp, mode);
+                  list_dir(rp, &options);
                   free(rp);
                }
           }
         fflush(stdout);
-        exit(0);
 //        ecore_main_loop_begin();
         ecore_evas_free(ee);
      }
